/*******************************************************************************
 Project name:  XTAL_TESTER_STM32

 c/ Blue Pill (STM32F103C8) acoplado a um LCD 16x2

 Autor: Claudio Lrios

 Data:27/04/2021

 Objetivo: Testar funcionamento de cristais osciladores. 
 O cristal  colocado em um circuito oscilador baseado no circuito
 integrado SN74HC04N, sendo a frequncia de sua sada mostrado no display
 LCD.
  
 Compilador MikroC PRO for ARM V.6.2.0 (VERSO GRATUITA E LIMITADA)

 Use bibliotecas LCD,LCD_CONSTANTS nas opes da Library Manager.

 Freq=72MHZ (XTAL8MHZ + PLL)

 BLOG PICSOURCE

 Uso didtico apenas

 ******************************************************************************/


// LCD module connections
sbit LCD_RS at GPIOA_ODR.B5;
sbit LCD_EN at GPIOA_ODR.B4;
sbit LCD_D4 at GPIOA_ODR.B0;
sbit LCD_D5 at GPIOA_ODR.B1;
sbit LCD_D6 at GPIOA_ODR.B2;
sbit LCD_D7 at GPIOA_ODR.B3;
#define _GPIO_PINMASK_MY_LCD 0x3f

unsigned int unsigned int freq_lida,freq_lida1,cnt;
long freq;
char flag_fim=0;


/*******************************************************************************
   Rotina de apresentao da frequencia no display LCD 16X2
*******************************************************************************/
void Mostra_Freq_LCD() {
  unsigned  int a,b;
   Lcd_Out(2,1," F=            ");   // Escreve na linha 2
       Lcd_Cmd(0xc3);//2linha 3 coluna

       freq= (long) freq_lida1 << 16;
       freq= freq + freq_lida;


       a=freq/10000000;
       freq=freq%10000000;
       if(a>0)
       Lcd_Chr_CP(a + '0'); //imprime digito
       else
       Lcd_Chr_CP(' '); //imprime espao

       a=freq/1000000;
       freq=freq%1000000;
       Lcd_Chr_CP(a + '0'); //imprime digito

       Lcd_Chr_CP('.');//imprime ponto

       a=freq/100000;
       freq=freq%100000;
       Lcd_Chr_CP(a + '0'); //imprime digito

       a=freq/10000;
       freq=freq%10000;
       Lcd_Chr_CP(a + '0'); //imprime digito

       a=freq/1000;
       freq=freq%1000;
       Lcd_Chr_CP(a + '0'); //imprime digito

       a=freq/100;
       freq=freq%100;
       Lcd_Chr_CP(a + '0'); //imprime digito

       a=freq/10;
       freq=freq%10;
       Lcd_Chr_CP(a + '0'); //imprime digito
       Lcd_Chr_CP(freq + '0');//imprime digito


       Lcd_Chr_CP(' ');  //espao
       Lcd_Chr_CP('M');  //h
       Lcd_Chr_CP('H');  //h
       Lcd_Chr_CP('Z');  //z

  }
 /**********************************************************************
            Rotina de leitura da frequncia gerada pelo cristal em teste
 **********************************************************************/
  void Le_Freq(){
 //  TIM1_ARR=0XFFFF;
 //  TIM1_RCR=0;
//   TIM2_ARR=0XFFFF;
   TIM2_CNT=0; //reseta contador do timer 2
   TIM1_CNT =0; //reseta contador do timer 1
   TIM1_CR1.CEN = 1;             // habilita timer 1
   TIM2_CR1.CEN = 1;             // habilita timer 2
   delay_ms(1000);               //delay de 1 segundo
   TIM1_CR1.CEN = 0;             // Disabilita timer 1
   TIM2_CR1.CEN = 0;             // Disabilita timer 2
   freq_lida1= TIM2_CNT;         //salva resultado high
   freq_lida=TIM1_CNT;           //salva resultado low
  }
  
/********************************************************************************
   Rotina Main e Set ups
*******************************************************************************/
void main() {

  GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_MY_LCD ); // Direo dos pinos LCD =sadas
  GPIO_Digital_Input(&GPIOA_BASE, _GPIO_PINMASK_9 );// entrada do frequencimetro (A9)

  RCC_APB2ENR.TIM1EN = 1;  //habilita clock para o timer1
  RCC_APB1ENR.TIM2EN = 1;  //habilita clock para o timer2
  TIM1_CCMR1_Input=0B1000000000;//seleciona filtro no clock externa pino A9, ext modo1
  TIM1_CCER.CC2P=0; //borda de subida
  TIM1_SMCR=0B1100111; //seleciona fonte de clock externa pino A9, ext modo1

  TIM1_CR2bits.MMS=0B010;//modo mestre com trigger no update (overflow do timer)
  TIM2_SMCRbits.TS=0b000;//sada do timer1 como trigger do timer 2
  TIM2_SMCRbits.SMS=0B111; //clock externo mode1 no trigger (contar pulsos)

  Lcd_Init();    // Inicializa o LCD
  Lcd_Cmd(_LCD_CURSOR_OFF);//desliga cursor
  Lcd_Cmd(_LCD_CLEAR);  //limpa a tela
  Lcd_Out(1,1," Xtal_Tester_STM");   // Escreve nome do aparelho
  Lcd_Out(2,1," by PicSource");   // Escreve nome do blog
  delay_ms(2000);// Tempo de apresentao da mensagem

/*******************************************************************************
  Loop principal
*******************************************************************************/
  for(;;) {
    Le_Freq();//l frequencia gerada pelo cristal em teste
    Mostra_Freq_LCD();// Formata e mostra resultado no LCD 16x2
  }
}